package edu.ncsu.csc.mas.enactments.JessJ2EE;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;



/**
 * Represents a message exchanged in the Jess/J2EE enactment of OWLP. It would be
 * passed as an <code>Object</code> in a JMS <code>ObjectMessage</code> hence it
 * implements <code>Serializable</code>.
 * 
 * @author Nirmit Desai
 *
 * 
 */

public class JJMessage implements Serializable{
	
	/**
	 * Uniquely identifies the protocol to which the message belongs
	 */
	protected int protID;
	
	/**
	 * Uniquely identifies the instance of the protocol, i.e., the conversation to which
	 * the message belongs
	 */
	protected int convID;
	/**
	 * One conversation of the protocol may have multiple transactions, this identifies the transaction
	 */
	protected int transID;
	/**
	 * Identifies the message's sequence number in the conversation
	 */
	protected static int seqNum=0;	
	
	/*
	 * Name of the sender role
	 */
	protected String senderName;
	
	/*
	 * Name of the receiver role
	 */
	protected String receiverName;
	
	/*
	 * Type of the message (name of the OWLP message individual)
	 */
	protected String type;
	
	public JJMessage(int transID, String type)
	{
		this.transID=transID;
		this.type=type;
		seqNum++;
	}
	
	/**
	 * @return Returns the convID.
	 */
	public int getConvID() {
		return convID;
	}
	/**
	 * @param convID The convID to set.
	 */
	public void setConvID(int convID) {
		this.convID = convID;
	}
	/**
	 * @return Returns the protID.
	 */
	public int getProtID() {
		return protID;
	}
	/**
	 * @param protID The protID to set.
	 */
	public void setProtID(int protID) {
		this.protID = protID;
	}
	/**
	 * @return Returns the seqNum.
	 */
	public int getSeqNum() {
		return seqNum;
	}
	/**
	 * @param seqNum The seqNum to set.
	 */
	public static void setSeqNum(int seqNum1) {
		seqNum = seqNum1;
	}
	/**
	 * @return Returns the transID.
	 */
	public int getTransID() {
		return transID;
	}
	/**
	 * @param transID The transID to set.
	 */
	public void setTransID(int transID) {
		this.transID = transID;
	}

	
	public void writeObject(ObjectOutputStream s) throws IOException {
		s.defaultWriteObject();
		// customized serialization code
	}	
	
    public void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
		s.defaultReadObject();
		// customized deserialization code
		
		// followed by code to update the object, if necessary
	}
    
	public String getReceiverName() {
		return receiverName;
	}
	public void setReceiverName(String receiver) {
		this.receiverName = receiver;
	}
	public String getSenderName() {
		return senderName;
	}
	public void setSenderName(String sender) {
		this.senderName = sender;
	}
	/**
	 * @return Returns the type.
	 */
	public String getType() {
		return type;
	}
	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;
	}
}
