/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.* ;

/**
 * Add a proposition to the state of the protocol.
 * 
 * In <a href="http://bombadil.csc.ncsu.edu/owl/v3_6/Protocol.owl">OWL-P</a>, each message
 * has zero or more effects. Each effect is an action. THe AddProposition action adds a
 * proposition to the current state of the protocol. Protocol states are sets of 
 * propositions.
 * 
 * @author Ashok Mallya
 *
 */
public class AddProposition extends Action {
    
    private Proposition proposition ;

    /**
     * Return a fresh <code>AddProposition</code> object
     *
     */
    public AddProposition() {
    }
    
    /**
     * Construct a fresh <code>AddProposition</code> object, given the proposition that it 
     * is to add to the protocol state.
     * 
     * @param res The proposition to be added to the protocol state.
     */
    public AddProposition(OntResource res) {
       this.proposition = new Proposition(res) ;
    }
    
    public String toString() {
        String retString = new String("addProposition(" ) ;
        if(this.proposition!=null) retString+= proposition.getContent() ;
        retString += ") " ;
        return retString;
    }

    /**
     * @return Returns the proposition.
     */
    public Proposition getProposition() {
        return proposition;
    }
    /**
     * @param proposition The proposition to set.
     */
    public void setProposition(Proposition proposition) {
        this.proposition = proposition;
    }
}
