/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.* ;

import edu.ncsu.csc.mas.ontology.exceptions.*;
import edu.ncsu.csc.mas.utils.OwlpGenerator;


/**
 * The condition class implements the condition of a commitment
 * 
 * @author Ashok Mallya
 * 
 */
public class Condition {
    
    private Proposition proposition ;


	/**
	 * Return a fresh <code>Condition</code> object.
	 * 
	 */
    public Condition (){
        super();
        this.proposition = null ;
    }
    
    /**
     * Construct a Condition given an instance of a condition in OWL-P .
     * 
     * The <code>OntClass</code> a proposition.
     * 
     * @param clas The OntClass object
     *  
     * @throws MalformedConditionException
     */
    public Condition(OntResource clas) 
    throws MalformedConditionException 
    {
        OwlpGenerator factory = OwlpGenerator.getOwlpGenerator() ;
        if(clas.getRDFType().getLocalName().equals("Proposition")) {
            if(clas.canAs(OntResource.class)) {
                this.proposition = new Proposition((OntResource)clas.as(OntResource.class)) ;
            }
        } else return ;
    }
    
    public Condition(Condition c) {
            this.proposition = new Proposition(c.getProposition()) ;
    }

    public String toString() {
        String retString = new String() ;
        retString = this.proposition.toJessStringForCondition() ;
        
        return retString ;
        
    }
    
    
    
    /**
     * Return the proposition, of this condition is a proposition
     * 
     * @return Returns the proposition.
     */
    public Proposition getProposition() {
            return proposition;
    }
    
    /**
     * Set the proposition if this condition is a proposition
     * 
     * @param proposition The proposition to set. 
     * @throws MalformedConditionException
     */
    public void setProposition(Proposition proposition)
    throws MalformedConditionException
    {
        //if this condition is not a proposition, throw an exception
        if(this.proposition==null) {
            this.proposition = new Proposition() ;
            this.proposition.setContent(proposition.getContent()) ;
        } else {
            this.proposition.setContent(proposition.getContent()) ;
        }
    }
}
