/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.RDFNode;

import edu.ncsu.csc.mas.ontology.exceptions.JenaConversionException;
import edu.ncsu.csc.mas.ontology.exceptions.MalformedConditionException;
import edu.ncsu.csc.mas.utils.Constants;
import edu.ncsu.csc.mas.utils.OwlpGenerator;

/**
 * A conditional commitment made by one role to another.
 * 
 * A conditional commitment has a pre-condition. When this pre-condition has the value <code>true</code>
 * the conditional commitment becomes a (base-level) commitment
 * 
 * @author Ashok Mallya
 *
 */
public class ConditionalCommitment extends Commitment {
    private Condition preCondition ;

    /**
     * Return a fresh <code>ConditionalCommitment</code> object
     */
    public ConditionalCommitment() {
    }
    
    /**
     * Return a fresh <code>ConditionalCommitment</code> object given the JENA object 
     * representing it.
     * 
     * @param res the JENA object representing the commitment
     * @throws MalformedConditionException
     * @throws JenaConversionException
     */
    public ConditionalCommitment(OntResource res) 
    throws MalformedConditionException, 
    JenaConversionException
    {
        super(res) ;
        OwlpGenerator factory = OwlpGenerator.getOwlpGenerator();
        RDFNode resRDFNode = res.getPropertyValue(factory.getBaseModel().getProperty(Constants.OntologyFileLocation + Constants.hash + "hasPrecondition")) ;
        if(resRDFNode.canAs(OntResource.class)) {
            this.preCondition = new Condition((OntResource)resRDFNode.as(OntResource.class)) ;
        }
        /**
        for(sIter = res.listProperties(); sIter.hasNext();) {
            Statement st = sIter.nextStatement() ;
            Property prop = st.getPredicate();
            if(prop.getLocalName().equals("hasPrecondition")) {
                RDFNode obj = st.getObject() ;
                OntResource objRes ;
                if(obj.canAs(OntResource.class)) 
                    objRes = (OntResource)obj ;
                else
                    continue ;
                this.preCondition = new Condition(objRes) ;
            }
        }
        **/
    }

    public String toString() {
        String retString = new String("CC(");
        if(this.debtor!= null)
            retString += this.debtor.toString() + ", ";
        if(this.creditor!= null)
            retString += this.creditor.toString() + ", ";
        if(this.preCondition!= null)
            retString += this.preCondition.toString() + ", ";
        if(this.condition!= null)
            retString += this.condition.toString() + ") "; 
        
        return retString ;
    }
    /**
     * @author pchakra 
     *
     */
    public String toJessString(int num){
    	String retString = "";
    	retString+= ("\n\t(bind ?cc-var(new JJConditionalCommitment \"" + this.debtor.toString()+ "\"   \""+ this.creditor.toString()+"\"   \""  +this.preCondition.toString() + "\" \"" + this.condition.toString() + "\" ?t" + num +"))");
    	retString+=("\n\t (definstance ccom ?cc-var static)");
    	return retString;
       
    }
    
}
