/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.*;
import java.util.*;
import com.hp.hpl.jena.rdf.model.*;

import edu.ncsu.csc.mas.ontology.exceptions.JenaConversionException;
import edu.ncsu.csc.mas.ontology.exceptions.MalformedConditionException;
import edu.ncsu.csc.mas.utils.Constants;
import edu.ncsu.csc.mas.utils.OwlpGenerator;

/**
 * @author aumallya, pchakra
 *
 */
public class Rule {

	private Vector body;

	private Vector head;

	private String name;

	private StmtIterator sIter;

	private Vector bodySlots;

	private Vector headSlots;

	private Policy policy;
	
	Vector returnSlots;
	

	public Rule() {
		this.body = new Vector();
		this.head = new Vector();
		this.name = new String();
		this.bodySlots = new Vector();
		this.headSlots = new Vector();
		this.returnSlots = new Vector();

	}

	public Rule(Rule r) {
		this.body = (Vector) r.getBody().clone();
		this.head = (Vector) r.getHead().clone();
		this.name = new String(r.getName());
		this.bodySlots = new Vector();
		this.headSlots = new Vector();
		this.returnSlots = new Vector();
	}

	public Rule(String name, Vector ruleBody, Vector ruleHead) {
		this.name = name;
		this.head = ruleHead;
		this.body = ruleBody;
		this.bodySlots = new Vector();
		this.headSlots = new Vector();
		this.returnSlots = new Vector();
	}

	/**
	 * Create a rule from a given resource
	 * 
	 * @param ontRule the resource (a swrl implication)
	 */
	public Rule(OntResource ontRule) {
		if (Constants.DEBUG)
			System.out.println("OntRule = " + ontRule);
		this.name = ontRule.getLocalName();
		System.out.println("LocalName of ontRule: " + name);
		OwlpGenerator factory = OwlpGenerator.getOwlpGenerator();
		for (sIter = ontRule.listProperties(); sIter.hasNext();) {
			//get the next statement in which this rule is the subject
			Statement ruleSt = sIter.nextStatement();
			//     System.out.println("Statement: " + ruleSt );
			//get the object of the statement
			RDFNode objRDFNode = ruleSt.getObject();
			//      System.out.println("RDFNode: " + objRDFNode );
			OntResource ruleObj;
			//try and convert this object into an OntResource
			if (objRDFNode.canAs(OntResource.class)) {
				ruleObj = (OntResource) objRDFNode.as(OntResource.class);
			} else
				continue;
			//Now see what property relates the rule to the object
			if (ruleSt.getPredicate().getLocalName().equals("head")) {
				//the property relates this rule to its head
				//the ruleObj is an AtomList.
				// we need to create a vector of Atoms

				if (this.head == null)
					this.head = new Vector();

				RDFNode firstRDFNode, restRDFNode;
				OntResource firstResource, restResource;
				do {
					firstRDFNode = ruleObj.getPropertyValue(factory
							.getBaseModel().getProperty(
									Constants.RDFSyntaxLocation
											+ Constants.hash + "first"));

					restRDFNode = ruleObj.getPropertyValue(factory
							.getBaseModel().getProperty(
									Constants.RDFSyntaxLocation
											+ Constants.hash + "rest"));
					if (firstRDFNode.canAs(OntResource.class)) {
						firstResource = (OntResource) firstRDFNode
								.as(OntResource.class);

						SwrlIndividualPropertyAtom swrlAtom = new SwrlIndividualPropertyAtom(
								firstResource);

						this.head.add(swrlAtom);

						//if(Constants.DEBUG==true) System.out.println("first is = " + firstResource) ;
					}
					if (restRDFNode == null)
						break;
					if (restRDFNode.canAs(OntResource.class)) {
						ruleObj = (OntResource) restRDFNode
								.as(OntResource.class);
						//if(Constants.DEBUG==true) System.out.println("rest is = " + ruleObj) ;
					}

				} while (ruleObj.toString().equals(
						Constants.RDFSyntaxLocation + Constants.hash + "nil") != true);

				for (int i = 0; i < head.size(); i++) {
					SwrlIndividualPropertyAtom swrlAtom = (SwrlIndividualPropertyAtom) head
							.elementAt(i);
					/*If the swrlAtom has the predicate sendMessage then there is a need to check whether the messageProp has been asserted to the KB or not 
					 * Hence it looks for all propositions which have been asserted to the KB to check if the messageProp exists or not
					 * If it does then that proposition is removed from the KB since the sendMessage takes care of it. 
					 * 
					 * */

					if (swrlAtom.getOwlpPropertyPredicate().getName().equals(
							"sendMessage")) {
						Message m = (Message) swrlAtom.getOwlpArgument2();
						System.out.println("entered Message " + m);
						for (int j = 0; j < head.size(); j++) {
							SwrlIndividualPropertyAtom sAtom = (SwrlIndividualPropertyAtom) head
									.elementAt(j);
							if (sAtom.getOwlpPropertyPredicate().getName()
									.equals("assertToKB")) {
								System.out.println("found assertToKB");
								if (sAtom.getOwlpArgument1() instanceof Proposition) {
									Proposition p = (Proposition) sAtom
											.getOwlpArgument1();

									System.out.println("comparing "
											+ p.getContent() + " "
											+ m.getType());
									if (p.getContent().equals(
											m.getType() + "Prop"))
										head.removeElementAt(j);

								}

							}

						}
					}

				}

			}

			if (ruleSt.getPredicate().getLocalName().equals("body")) {
				//the property relates this rule to its body
				//the ruleObj is an AtomList.
				// we need to create a vector of Atoms

				if (this.body == null)
					this.body = new Vector();

				RDFNode firstRDFNode, restRDFNode;
				OntResource firstResource, restResource;
				do {
					firstRDFNode = ruleObj.getPropertyValue(factory
							.getBaseModel().getProperty(
									Constants.RDFSyntaxLocation
											+ Constants.hash + "first"));
					restRDFNode = ruleObj.getPropertyValue(factory
							.getBaseModel().getProperty(
									Constants.RDFSyntaxLocation
											+ Constants.hash + "rest"));
					if (firstRDFNode.canAs(OntResource.class)) {
						firstResource = (OntResource) firstRDFNode
								.as(OntResource.class);
						SwrlIndividualPropertyAtom swrlAtom = new SwrlIndividualPropertyAtom(
								firstResource);

						this.body.add(swrlAtom);

						//if(Constants.DEBUG) System.out.println("first is = " + firstResource) ;
					}
					if (restRDFNode == null)
						break;
					if (restRDFNode.canAs(OntResource.class)) {
						ruleObj = (OntResource) restRDFNode
								.as(OntResource.class);
						// if(Constants.DEBUG) System.out.println("rest is = " + ruleObj) ;
					}

				} while (ruleObj.toString().equals(
						Constants.RDFSyntaxLocation + Constants.hash + "nil") != true);

			}
		}
	}

	/**
	 * @return Returns the head.
	 */
	public Vector getHead() {
		return head;
	}

	/**
	 * @param head The head to set.
	 */
	public void setHead(Vector effects) {
		this.head = effects;
	}

	public void addtoHead(SwrlIndividualPropertyAtom s) {
		if (this.head == null)
			this.head = new Vector();
		this.head.add(s);

	}

	/**
	 * @return Returns the body.
	 */
	public Vector getBody() {
		return body;
	}

	/**
	 * @param body The body to set.
	 */
	public void setBody(Vector preCondition) {
		this.body = (Vector) preCondition.clone();
	}

	public String toString(Protocol prot) {

		String retString = new String("\n\n(defrule " + name + "\n");

		SwrlIndividualPropertyAtom satomBody;

		if (body.size() > 1) {
			retString += "(and";
			for (int i = 0; i < body.size(); i++) {

				if ((body.elementAt(i)) instanceof SwrlIndividualPropertyAtom) {
					satomBody = (SwrlIndividualPropertyAtom) body.elementAt(i);
					retString += satomBody.toJessRuleString(i + 1, prot);

				}

				else if ((body.elementAt(i)) instanceof MessageInQueue) {
					MessageInQueue msgInQBody = (MessageInQueue) body
							.elementAt(i);
					retString += msgInQBody.toJessRuleString(i + 1);

				} else if ((body.elementAt(i)) instanceof Policy)
					retString += ((Policy) body.elementAt(i)).toString();

			}

			retString += ")";
			retString += "\n";

		} else if (body.size() == 1) {
			satomBody = (SwrlIndividualPropertyAtom) body.elementAt(0);
			retString += satomBody.toJessRuleString(1, prot);

		}

		retString += "=>\t ";

		int num;
		if (body.size() == 1)
			num = body.size();
		else
			num = body.size() - 1;

		if (head == null)
			return retString + ")";

		else if (head.size() == 1) {

			SwrlIndividualPropertyAtom satomHead = (SwrlIndividualPropertyAtom) head
					.elementAt(0);

			retString += satomHead.toJessRuleString(num, prot);

		} else {
			for (int i = 0; i < head.size(); i++) {

				SwrlIndividualPropertyAtom satomHead = (SwrlIndividualPropertyAtom) head
						.elementAt(i);
				retString += satomHead.toJessRuleString(num, prot);

			}
		}

		return retString + ")";

	}

	public String generatePolicy(Protocol prot, Role role) {
		String policyName = "";

		int index = 0;

		if (this.name.indexOf("Rule") != -1)
			index = this.name.indexOf("Rule");

		else if (this.name.indexOf("_rule") != -1)
			index = this.name.indexOf("_rule");

		policyName = this.name.substring(0, index);

		String retString = new String("\n(defrule " + policyName + "_Policy \n");
		int num = 0;
		

		if (body.size() > 1) {
			retString += "(and";
			for (int i = 0; i < body.size(); i++) {
				if ((body.elementAt(i)) instanceof SwrlIndividualPropertyAtom) {
					SwrlIndividualPropertyAtom satom = (SwrlIndividualPropertyAtom) body
							.elementAt(i);

					retString += satom.toJessRuleString(i + 1, prot);
					Proposition p = new Proposition(satom.getOntArgument2());
					Vector slotsInpVector = new Vector();
					if (!(p.getSlots().isEmpty())) {
						slotsInpVector = p.getSlots();
						this.returnSlots.addAll(slotsInpVector);
					}

					num++;
				}

				else if ((body.elementAt(i)) instanceof MessageInQueue) {
					MessageInQueue mq = (MessageInQueue) body.elementAt(i);

					retString += mq.toJessRuleString(i + 1);

					Vector slotsInmVector = new Vector();
					slotsInmVector = mq.getSlots();
					this.returnSlots.removeAllElements();
					this.returnSlots.addAll(slotsInmVector);

					num++;
				}

				else if ((body.elementAt(i)) instanceof Policy) {

					if (num == 1) {

						int ind = retString.indexOf("(and");
						retString = retString.substring(0, ind)
								+ retString.substring(ind + 4, retString
										.length() - 2);

					}

				}

			}

			retString += ")";
		} else if (body.size() == 1) {
			SwrlIndividualPropertyAtom satom = (SwrlIndividualPropertyAtom) body
					.elementAt(0);
			retString += satom.toJessRuleString(1, prot);
			Proposition p = new Proposition(satom.getOntArgument2());
			Vector slotsInpVector = new Vector();
			if (!(p.getSlots().isEmpty())) {
				slotsInpVector = p.getSlots();
				this.returnSlots.addAll(slotsInpVector);
			}

			num++;
		}

		retString += "\n =>\t ";
		retString += "(call (new " + role.getName() + prot.getInstName()
				+ "Policy)  " + policyName + "Method ";

		for (int j = 0; j < returnSlots.size(); j++) {

			Slot slot = (Slot) returnSlots.elementAt(j);

			retString += " ?" + slot.toJessString() + "-var" + num;

		}
		retString += ")\n";

		return retString;

	}
	
	

	/**
	 * @return Returns the returnSlots.
	 */
	public Vector getReturnSlots() {
		return returnSlots;
	}
	/**
	 * @param returnSlots The returnSlots to set.
	 */
	public void setReturnSlots(Vector returnSlots) {
		this.returnSlots = returnSlots;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	public void removeAtom(SwrlIndividualPropertyAtom swrlAtom, int bodyOrHead) {
		if (bodyOrHead == Constants.BODY) {
			if (this.body == null)
				return;
			for (int i = 0; i < this.body.size(); i++) {
				if (((SwrlIndividualPropertyAtom) this.body.elementAt(i))
						.equals(swrlAtom))
					this.body.removeElementAt(i);
			}
		} else if (bodyOrHead == Constants.HEAD) {
			if (this.head == null)
				return;
			for (int i = 0; i < this.head.size(); i++) {
				if (((SwrlIndividualPropertyAtom) this.head.elementAt(i))
						.equals(swrlAtom))
					this.head.removeElementAt(i);
			}

		}
	}

	public void addToBody(Vector swrlAtomVec) {
		this.body.addAll(swrlAtomVec);
	}

	public void addToBody(MessageInQueue mq) {
		this.body.add(mq);

	}

	public void addToBody(Policy policy) {
		this.body.add(policy);
	}

	public Vector getBodySlots() {
		SwrlIndividualPropertyAtom satomBody;
		if (body.size() > 1) {

			for (int i = 0; i < body.size(); i++) {

				if ((body.elementAt(i)) instanceof SwrlIndividualPropertyAtom) {
					satomBody = (SwrlIndividualPropertyAtom) body.elementAt(i);
					if (!satomBody.getSlots().isEmpty())
						this.bodySlots.addAll(satomBody.getSlots());
				}

				else if ((body.elementAt(i)) instanceof MessageInQueue) {
					MessageInQueue msgInQBody = (MessageInQueue) body
							.elementAt(i);
					if (!msgInQBody.getSlots().isEmpty())
						this.bodySlots.addAll(msgInQBody.getSlots());

				}
			}

		} else if (body.size() == 1) {
			satomBody = (SwrlIndividualPropertyAtom) body.elementAt(0);
			if (!satomBody.getSlots().isEmpty())
				this.bodySlots.addAll(satomBody.getSlots());
		}

		return this.bodySlots;
	}

	public Vector getHeadSlots() {
		SwrlIndividualPropertyAtom satomHead;
		if (head.size() == 1) {
			satomHead = (SwrlIndividualPropertyAtom) head.elementAt(0);
			if (!satomHead.getSlots().isEmpty())
				this.headSlots.addAll(satomHead.getSlots());

		}
		for (int i = 0; i < head.size(); i++) {

			satomHead = (SwrlIndividualPropertyAtom) head.elementAt(i);
			if (!satomHead.getSlots().isEmpty())
				this.headSlots.addAll(satomHead.getSlots());
		}
		return this.headSlots;
	}

}
